"use strict";

exports.__esModule = true;
exports.unpackSDKMeta = unpackSDKMeta;

var _url = _interopRequireDefault(require("url"));

var _sdkConstants = require("@paypal/sdk-constants");

var _jsxPragmatic = require("jsx-pragmatic");

var _constants = require("./constants");

var _util = require("./util");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function validatePaymentsSDKUrl({
  pathname,
  query,
  hash
}) {
  if (pathname !== _sdkConstants.SDK_PATH) {
    throw new Error(`Invalid path for sdk url: ${pathname || 'undefined'}`);
  } // $FlowFixMe


  for (const [key, val] of (0, _util.entries)(query)) {
    // $FlowFixMe
    if (!(0, _util.constHas)(_sdkConstants.SDK_QUERY_KEYS, key)) {
      throw new Error(`Unexpected query key for sdk url: ${key}`);
    }

    if (!val) {
      throw new Error(`Unexpected empty query value for sdk url: ${key}`);
    }

    if (typeof val !== 'string') {
      throw new TypeError(`Unexpected non-string key for sdk url: ${key}`);
    }

    if (!val.match(/^[a-zA-Z0-9_,-@.]+$/) && !val.match(/^\*$/)) {
      throw new Error(`Unexpected characters in query key for sdk url: ${key}=${val}`);
    }
  }

  if (hash) {
    throw new Error(`Expected no hash to be passed in sdk url, got ${hash}`);
  }
}

function validateLegacySDKUrl({
  pathname
}) {
  if (!pathname.match(_constants.LEGACY_SDK_PATH)) {
    throw new Error(`Invalid path for legacy sdk url: ${pathname || 'undefined'}`);
  }
}

function isLegacySDKUrl(hostname, pathname) {
  if (hostname === _constants.HOST.PAYPALOBJECTS) {
    return true;
  }

  if (hostname.endsWith(_constants.HOST.PAYPAL) && pathname.match(_constants.LEGACY_SDK_PATH)) {
    return true;
  }

  return false;
}

function isSDKUrl(hostname) {
  if (hostname.endsWith(_constants.HOST.PAYPAL)) {
    return true;
  }

  return false;
}

function validateSDKUrl(sdkUrl) {
  const {
    protocol,
    hostname,
    pathname,
    query,
    hash
  } = _url.default.parse(sdkUrl, true);

  if (!hostname) {
    throw new Error(`Expected host to be passed for sdk url`);
  }

  if (!pathname) {
    throw new Error(`Expected pathname for sdk url`);
  }

  if (protocol !== _constants.PROTOCOL.HTTP && protocol !== _constants.PROTOCOL.HTTPS) {
    throw new Error(`Expected protocol for sdk url to be ${_constants.PROTOCOL.HTTP} or ${_constants.PROTOCOL.HTTPS} for host: ${hostname} - got ${protocol || 'undefined'}`);
  }

  if (isLegacySDKUrl(hostname, pathname)) {
    validateLegacySDKUrl({
      pathname
    });
  } else if (isSDKUrl(hostname)) {
    if (hostname !== _constants.HOST.LOCALHOST && protocol !== _constants.PROTOCOL.HTTPS) {
      throw new Error(`Expected protocol for sdk url to be ${_constants.PROTOCOL.HTTPS} for host: ${hostname} - got ${protocol || 'undefined'}`);
    }

    if (sdkUrl.match(/&{2,}/) || sdkUrl.match(/&$/)) {
      throw new Error(`Expected sdk url to not contain double ampersand or end in ampersand`);
    }

    validatePaymentsSDKUrl({
      protocol,
      hostname,
      pathname,
      query,
      hash
    });
  } else {
    throw new Error(`Expected host to be a subdomain of ${_constants.HOST.PAYPAL} or ${_constants.HOST.PAYPALOBJECTS}`);
  }
}

function validateHost(url) {
  const {
    host,
    hostname
  } = _url.default.parse(`https://${url}`, true);

  if (url !== host) {
    throw new Error(`Expected only host to be passed, got ${url}`);
  }

  if (!hostname || !hostname.endsWith(_constants.HOST.PAYPAL)) {
    throw new Error(`Expected a paypal host`);
  }
}

function getSDKScriptAttributes(sdkUrl, allAttrs) {
  const attrs = {};

  if (sdkUrl) {
    const {
      hostname,
      pathname
    } = _url.default.parse(sdkUrl, true);

    if (!hostname) {
      throw new Error(`Expected host to be passed for sdk url`);
    }

    if (!pathname) {
      throw new Error(`Expected pathname for sdk url`);
    }

    if (isLegacySDKUrl(hostname, pathname)) {
      attrs[_constants.DATA_ATTRIBUTES.PAYPAL_CHECKOUT] = true;
      attrs[_constants.DATA_ATTRIBUTES.NO_BRIDGE] = true;
    }
  } // only those in SDK_SETTINGS


  const validAttrs = Object.values(_sdkConstants.SDK_SETTINGS);

  for (const key in allAttrs) {
    if (validAttrs.includes(key)) {
      attrs[key] = allAttrs[key];
    }
  }

  return attrs;
}

function unpackSDKMeta(sdkMeta) {
  const {
    url,
    attrs
  } = sdkMeta ? JSON.parse(Buffer.from(sdkMeta, 'base64').toString('utf8')) : _constants.DEFAULT_SDK_META;

  if (url) {
    validateSDKUrl(url);
  }

  if (attrs && attrs[_sdkConstants.SDK_SETTINGS.STAGE_HOST]) {
    validateHost(attrs[_sdkConstants.SDK_SETTINGS.STAGE_HOST]);
  }

  if (attrs && attrs[_sdkConstants.SDK_SETTINGS.API_STAGE_HOST]) {
    validateHost(attrs[_sdkConstants.SDK_SETTINGS.API_STAGE_HOST]);
  }

  const getSDKLoader = ({
    baseURL = _constants.DEFAULT_LEGACY_SDK_BASE_URL,
    nonce = ''
  } = {}) => {
    if (url) {
      const validAttrs = getSDKScriptAttributes(url, attrs);
      return (0, _jsxPragmatic.node)("script", _extends({
        nonce: nonce,
        src: url
      }, validAttrs)).render((0, _jsxPragmatic.html)());
    }

    return (0, _jsxPragmatic.node)("script", {
      nonce: nonce,
      innerHTML: `
                    (function() {
                        function loadScript(url, attributes) {
                            var scriptTag = '<scr' + 'ipt src="' + url + '" ' + (attributes || '') + '></scr' + 'ipt>';
                            document.write(scriptTag);
                        }

                        function loadV4() {
                            if (!window.name || window.name.indexOf('xcomponent') !== 0) {
                                return;
                            }

                            var version = window.name.split('__')[2].replace(/_/g, '.');

                            if (!version.match(/^[0-9a-zA-Z.]+$/)) {
                                return;
                            }

                            if (version === '4' || version === 'latest') {
                                version = '';
                            }

                            var url = '${baseURL}checkout' + (version ? ('.' + version) : '') + '.js';
                            var attributes = '${_constants.DATA_ATTRIBUTES.PAYPAL_CHECKOUT} ${_constants.DATA_ATTRIBUTES.NO_BRIDGE}';

                            loadScript(url, attributes);
                        }

                        function loadV5() {
                            var ancestor = (window.parent && window.parent !== window)
                                ? window.parent
                                : window.opener;

                            if (!ancestor || !ancestor.document) {
                                return;
                            }

                            var v5script = ancestor.document.querySelector('script[src*="/sdk/js"]');

                            if (!v5script || !v5script.src) {
                                return;
                            }

                            loadScript(v5script.src);
                        }

                        try {
                            if (window.paypal && window.paypal.version) {
                                return;
                            }

                            loadV4();
                            loadV5();
                        } catch (err) {
                            return;
                        }
                    })();
                `
    }).render((0, _jsxPragmatic.html)());
  };

  return {
    getSDKLoader
  };
}